// This file was autogenerated by closure-primitives/depswriter.py.
// Please do not edit.
goog.addDependency('../bin/zlib.pretty.js', ['USE_TYPEDARRAY', 'Zlib', 'Zlib.Adler32', 'Zlib.BitStream', 'Zlib.CRC32', 'Zlib.Deflate', 'Zlib.Gunzip', 'Zlib.GunzipMember', 'Zlib.Gzip', 'Zlib.Heap', 'Zlib.Huffman', 'Zlib.Inflate', 'Zlib.InflateStream', 'Zlib.RawDeflate', 'Zlib.RawInflate', 'Zlib.RawInflateStream', 'Zlib.Unzip', 'Zlib.Util', 'Zlib.Zip', 'Zlib.exportObject'], ['USE_TYPEDARRAY', 'Zlib', 'Zlib.Adler32', 'Zlib.BitStream', 'Zlib.CRC32', 'Zlib.Deflate', 'Zlib.Gunzip', 'Zlib.GunzipMember', 'Zlib.Gzip', 'Zlib.Heap', 'Zlib.Huffman', 'Zlib.Inflate', 'Zlib.InflateStream', 'Zlib.RawDeflate', 'Zlib.RawInflate', 'Zlib.RawInflateStream', 'Zlib.Unzip', 'Zlib.Util', 'Zlib.Zip', 'Zlib.exportObject']);
goog.addDependency('../closure-primitives/base.js', ['goog'], []);
goog.addDependency('../define/typedarray/hybrid.js', ['USE_TYPEDARRAY'], []);
goog.addDependency('../define/typedarray/use.js', ['USE_TYPEDARRAY'], []);
goog.addDependency('../src/adler32.js', ['Zlib.Adler32'], ['USE_TYPEDARRAY', 'Zlib.Util']);
goog.addDependency('../src/bitstream.js', ['Zlib.BitStream'], ['USE_TYPEDARRAY']);
goog.addDependency('../src/crc32.js', ['Zlib.CRC32'], ['USE_TYPEDARRAY']);
goog.addDependency('../src/deflate.js', ['Zlib.Deflate'], ['USE_TYPEDARRAY', 'Zlib', 'Zlib.Adler32', 'Zlib.RawDeflate']);
goog.addDependency('../src/export_object.js', ['Zlib.exportObject'], ['Zlib']);
goog.addDependency('../src/gunzip.js', ['Zlib.Gunzip'], ['USE_TYPEDARRAY', 'Zlib.CRC32', 'Zlib.GunzipMember', 'Zlib.Gzip', 'Zlib.RawInflate']);
goog.addDependency('../src/gunzip_member.js', ['Zlib.GunzipMember'], []);
goog.addDependency('../src/gzip.js', ['Zlib.Gzip'], ['USE_TYPEDARRAY', 'Zlib.CRC32', 'Zlib.RawDeflate']);
goog.addDependency('../src/heap.js', ['Zlib.Heap'], ['USE_TYPEDARRAY']);
goog.addDependency('../src/huffman.js', ['Zlib.Huffman'], ['USE_TYPEDARRAY']);
goog.addDependency('../src/inflate.js', ['Zlib.Inflate'], ['USE_TYPEDARRAY', 'Zlib.Adler32', 'Zlib.RawInflate']);
goog.addDependency('../src/inflate_stream.js', ['Zlib.InflateStream'], ['USE_TYPEDARRAY', 'Zlib', 'Zlib.RawInflateStream']);
goog.addDependency('../src/rawdeflate.js', ['Zlib.RawDeflate'], ['USE_TYPEDARRAY', 'Zlib.BitStream', 'Zlib.Heap']);
goog.addDependency('../src/rawinflate.js', ['Zlib.RawInflate'], ['USE_TYPEDARRAY', 'Zlib.Huffman']);
goog.addDependency('../src/rawinflate_stream.js', ['Zlib.RawInflateStream'], ['USE_TYPEDARRAY', 'Zlib.Huffman']);
goog.addDependency('../src/unzip.js', ['Zlib.Unzip'], ['USE_TYPEDARRAY', 'Zlib.CRC32', 'Zlib.RawInflate', 'Zlib.Zip']);
goog.addDependency('../src/util.js', ['Zlib.Util'], []);
goog.addDependency('../src/zip.js', ['Zlib.Zip'], ['USE_TYPEDARRAY', 'Zlib.CRC32', 'Zlib.RawDeflate']);
goog.addDependency('../src/zlib.js', ['Zlib'], []);
